@@ -6,6 +6,7 @@
 # Revisions:    12/23/2005 - P. Harvey Created
 #
 # References:   1) http://www.microsoft.com/windows/windowsmedia/format/asfspec.aspx
+#               2) http://www.adobe.com/devnet/xmp/pdfs/XMPSpecificationPart3.pdf (Oct 2008)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::ASF;
@@ -14,12 +15,13 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
+use Image::ExifTool::RIFF;
 
-$VERSION = '1.05';
+$VERSION = '1.19';
 
 sub ProcessMetadata($$$);
 sub ProcessContentDescription($$$);
-sub ProcessPreview($$$);
+sub ProcessPicture($$$);
 sub ProcessCodecList($$$);
 
 # GUID definitions
@@ -62,9 +64,9 @@ my %advancedContentEncryption = (
 %Image::ExifTool::ASF::Main = (
     PROCESS_PROC => \&Image::ExifTool::ASF::ProcessASF,
     NOTES => q{
-        ASF format is used by Windows WMA and WMV files.  Tag ID's aren't listed
-        because they are huge 128-bit GUID's that would ruin the formatting of this
-        table.
+        The ASF format is used by Windows WMA and WMV files, and DIVX videos.  Tag
+        ID's aren't listed because they are huge 128-bit GUID's that would ruin the
+        formatting of this table.
     },
     '75B22630-668E-11CF-A6D9-00AA0062CE6C' => {
         Name => 'Header',
@@ -75,6 +77,10 @@ my %advancedContentEncryption = (
     'D6E229D3-35DA-11D1-9034-00A0C90349BE' => 'Index',
     'FEB103F8-12AD-4C64-840F-2A1D2F7AD48C' => 'MediaIndex',
     '3CB73FD0-0C4A-4803-953D-EDF7B6228F0C' => 'TimecodeIndex',
+    'BE7ACFCB-97A9-42E8-9C71-999491E3AFAC' => { #2
+        Name => 'XMP',
+        SubDirectory => { TagTable => 'Image::ExifTool::XMP::Main' },
+    },
 );
 
 # ASF header objects
@@ -157,14 +163,14 @@ my %advancedContentEncryption = (
     BannerImageData => {},
     BannerImageType => {},
     BannerImageURL => {},
-    Bitrate => {},
+    Bitrate => { PrintConv => 'ConvertBitrate($val)' },
     Broadcast => {},
     BufferAverage => {},
     Can_Skip_Backward => {},
     Can_Skip_Forward => {},
     Copyright => { Groups => { 2 => 'Author' } },
     CopyrightURL => { Groups => { 2 => 'Author' } },
-    CurrentBitrate => {},
+    CurrentBitrate => { PrintConv => 'ConvertBitrate($val)' },
     Description => {},
     DRM_ContentID => {},
     DRM_DRMHeader_ContentDistributor => {},
@@ -182,7 +188,7 @@ my %advancedContentEncryption = (
     DRM_LASignatureRootCert => {},
     DRM_LicenseAcqURL => {},
     DRM_V1LicenseAcqURL => {},
-    Duration => {},
+    Duration => { PrintConv => 'ConvertDuration($val)' },
     FileSize => {},
     HasArbitraryDataStream => {},
     HasAttachedImages => {},
@@ -200,7 +206,7 @@ my %advancedContentEncryption = (
     NSC_Name => {},
     NSC_Phone => {},
     NumberOfFrames => {},
-    OptimalBitrate => {},
+    OptimalBitrate => { PrintConv => 'ConvertBitrate($val)' },
     PeakValue => {},
     Rating => {},
     Seekable => {},
@@ -267,11 +273,11 @@ my %advancedContentEncryption = (
     ParentalRating => {},
     ParentalRatingReason => {},
     PartOfSet => {},
-    PeakBitrate => {},
+    PeakBitrate => { PrintConv => 'ConvertBitrate($val)' },
     Period => {},
     Picture => {
         SubDirectory => {
-            TagTable => 'Image::ExifTool::ASF::Preview',
+            TagTable => 'Image::ExifTool::ASF::Picture',
         },
     },
     PlaylistDelay => {},
@@ -312,40 +318,40 @@ my %advancedContentEncryption = (
     Year => { Groups => { 2 => 'Time' } },
 );
 
-%Image::ExifTool::ASF::Preview = (
-    PROCESS_PROC => \&ProcessPreview,
-    GROUPS => { 2 => 'Video' },
+%Image::ExifTool::ASF::Picture = (
+    PROCESS_PROC => \&ProcessPicture,
+    GROUPS => { 2 => 'Image' },
     0 => {
-        Name => 'PreviewType',
-        PrintConv => {
-            0 => 'Other picture type',
-            1 => '32x32 PNG file icon',
-            2 => 'Other file icon',
-            3 => 'Front album cover',
-            4 => 'Back album cover',
-            5 => 'Leaflet page',
-            6 => 'Media label',
-            7 => 'Lead artist, performer, or soloist',
-            8 => 'Artists or performers',
+        Name => 'PictureType',
+        PrintConv => { # (Note: Duplicated in ID3, ASF and FLAC modules!)
+            0 => 'Other',
+            1 => '32x32 PNG Icon',
+            2 => 'Other Icon',
+            3 => 'Front Cover',
+            4 => 'Back Cover',
+            5 => 'Leaflet',
+            6 => 'Media',
+            7 => 'Lead Artist',
+            8 => 'Artist',
             9 => 'Conductor',
-            10 => 'Band or orchestra',
+            10 => 'Band',
             11 => 'Composer',
-            12 => 'Lyricist or writer',
-            13 => 'Recording studio or location',
-            14 => 'Recording session',
+            12 => 'Lyricist',
+            13 => 'Recording Studio or Location',
+            14 => 'Recording Session',
             15 => 'Performance',
-            16 => 'Capture from movie or video',
-            17 => 'A bright colored fish',
+            16 => 'Capture from Movie or Video',
+            17 => 'Bright(ly) Colored Fish',
             18 => 'Illustration',
-            19 => 'Band or artist logo',
-            20 => 'Publisher or studio logo',
+            19 => 'Band Logo',
+            20 => 'Publisher Logo',
         },
     },
-    1 => 'PreviewMimeType',
-    2 => 'PreviewDescription',
+    1 => 'PictureMimeType',
+    2 => 'PictureDescription',
     3 => {
-        Name => 'PreviewImage',
-        ValueConv => '$self->ValidateImage(\$val,$tag)',
+        Name => 'Picture',
+        Binary => 1,
     },
 );
 
@@ -357,7 +363,7 @@ my %advancedContentEncryption = (
         Format => 'binary[16]',
         ValueConv => 'Image::ExifTool::ASF::GetGUID($val)',
     },
-    16 => { Name => 'FileSize',     Format => 'int64u' },
+    16 => { Name => 'FileLength',   Format => 'int64u' },
     24 => {
         Name => 'CreationDate',
         Format => 'int64u',
@@ -366,26 +372,27 @@ my %advancedContentEncryption = (
         ValueConv => q{ # (89 leap years between 1601 and 1970)
             my $t = $val / 1e7 - (((1970-1601)*365+89)*24*3600);
             return Image::ExifTool::ConvertUnixTime($t) . 'Z';
-        }
+        },
+        PrintConv => '$self->ConvertDateTime($val)',
     },
     32 => { Name => 'DataPackets',  Format => 'int64u' },
     40 => {
         Name => 'PlayDuration',
         Format => 'int64u',
         ValueConv => '$val / 1e7',
-        PrintConv => '"$val sec"',
+        PrintConv => 'ConvertDuration($val)',
     },
     48 => {
         Name => 'SendDuration',
         Format => 'int64u',
         ValueConv => '$val / 1e7',
-        PrintConv => '"$val sec"',
+        PrintConv => 'ConvertDuration($val)',
     },
     56 => { Name => 'Preroll',      Format => 'int64u' },
     64 => { Name => 'Flags',        Format => 'int32u' },
     68 => { Name => 'MinPacketSize',Format => 'int32u' },
     72 => { Name => 'MaxPacketSize',Format => 'int32u' },
-    76 => { Name => 'MaxBitrate',   Format => 'int32u' },
+    76 => { Name => 'MaxBitrate',   Format => 'int32u', PrintConv => 'ConvertBitrate($val)' },
 );
 
 %Image::ExifTool::ASF::StreamProperties = (
@@ -413,7 +420,7 @@ my %advancedContentEncryption = (
         Name => 'TimeOffset',
         Format => 'int64u',
         ValueConv => '$val / 1e7',
-        PrintConv => '"$val sec"',
+        PrintConv => '"$val s"',
     },
     48 => {
         Name => 'StreamNumber',
@@ -426,121 +433,8 @@ my %advancedContentEncryption = (
             Name => 'AudioCodecID',
             Format => 'int16u',
             PrintHex => 1,
-            # from http://www.iana.org/assignments/wave-avi-codec-registry
-            # and http://developers.videolan.org/vlc/vlc/doc/doxygen/html/codecs_8h-source.html
-            PrintConv => {
-                0x0001 => 'Microsoft PCM',
-                0x0002 => 'Microsoft ADPCM',
-                0x0003 => 'IEEE Float',
-                0x0004 => 'Cmpaq VSELP',
-                0x0005 => 'IBM CVSD',
-                0x0006 => 'Microsoft A-Law',
-                0x0007 => 'Microsoft Mu-Law',
-                0x0008 => 'DTS Coherent Acoustics 1',
-                0x0010 => 'OKI ADPCM',
-                0x0011 => 'Intel DVI ADPCM',
-                0x0012 => 'Videologic MediaSpace ADPCM',
-                0x0013 => 'Sierra ADPCM',
-                0x0014 => 'G.723 ADPCM',
-                0x0015 => 'DSP Solution DIGISTD',
-                0x0016 => 'DSP Solution DIGIFIX',
-                0x0017 => 'Dialogic OKI ADPCM',
-                0x0018 => 'MediaVision ADPCM',
-                0x0019 => 'HP CU',
-                0x0020 => 'Yamaha ADPCM',
-                0x0021 => 'Speech Compression Sonarc',
-                0x0022 => 'DSP Group True Speech',
-                0x0023 => 'Echo Speech EchoSC1',
-                0x0024 => 'Audiofile AF36',
-                0x0025 => 'APTX',
-                0x0026 => 'AudioFile AF10',
-                0x0027 => 'Prosody 1612',
-                0x0028 => 'LRC',
-                0x0030 => 'Dolby AC2',
-                0x0031 => 'GSM610',
-                0x0032 => 'MSNAudio',
-                0x0033 => 'Antex ADPCME',
-                0x0034 => 'Control Res VQLPC',
-                0x0035 => 'Digireal',
-                0x0036 => 'DigiADPCM',
-                0x0037 => 'Control Res CR10',
-                0x0038 => 'NMS VBXADPCM',
-                0x0039 => 'Roland RDAC',
-                0x003a => 'EchoSC3',
-                0x003b => 'Rockwell ADPCM',
-                0x003c => 'Rockwell Digit LK',
-                0x003d => 'Xebec',
-                0x0040 => 'Antex Electronics G.721',
-                0x0041 => 'G.728 CELP',
-                0x0042 => 'MSG723',
-                0x0045 => 'G.726 ADPCM',
-                0x0050 => 'MPEG',
-                0x0052 => 'RT24',
-                0x0053 => 'PAC',
-                0x0055 => 'MPEG Layer 3',
-                0x0059 => 'Lucent G.723',
-                0x0060 => 'Cirrus',
-                0x0061 => 'ESPCM',
-                0x0062 => 'Voxware',
-                0x0063 => 'Canopus Atrac',
-                0x0064 => 'G.726 ADPCM',
-                0x0066 => 'DSAT',
-                0x0067 => 'DSAT Display',
-                0x0069 => 'Voxware Byte Aligned',
-                0x0070 => 'Voxware AC8',
-                0x0071 => 'Voxware AC10',
-                0x0072 => 'Voxware AC16',
-                0x0073 => 'Voxware AC20',
-                0x0074 => 'Voxware MetaVoice',
-                0x0075 => 'Voxware MetaSound',
-                0x0076 => 'Voxware RT29HW',
-                0x0077 => 'Voxware VR12',
-                0x0078 => 'Voxware VR18',
-                0x0079 => 'Voxware TQ40',
-                0x0080 => 'Softsound',
-                0x0081 => 'Voxware TQ60',
-                0x0082 => 'MSRT24',
-                0x0083 => 'G.729A',
-                0x0084 => 'MVI MV12',
-                0x0085 => 'DF G.726',
-                0x0086 => 'DF GSM610',
-                0x0088 => 'ISIAudio',
-                0x0089 => 'Onlive',
-                0x0091 => 'SBC24',
-                0x0092 => 'Dolby AC3 SPDIF',
-                0x0097 => 'ZyXEL ADPCM',
-                0x0098 => 'Philips LPCBB',
-                0x0099 => 'Packed',
-                0x00FF => 'MPEG-4',
-                0x0100 => 'Rhetorex ADPCM',
-                0x0101 => 'BeCubed Software IRAT',
-                0x0111 => 'Vivo G.723',
-                0x0112 => 'Vivo Siren',
-                0x0123 => 'Digital G.723',
-                0x0160 => 'Windows Media v1',
-                0x0161 => 'Windows Media v2',
-                0x0162 => 'Windows Media 9 Professional',
-                0x0163 => 'Windows Media 9 Lossless',
-                0x0200 => 'Creative ADPCM',
-                0x0202 => 'Creative FastSpeech8',
-                0x0203 => 'Creative FastSpeech10',
-                0x0220 => 'Quarterdeck',
-                0x0300 => 'FM Towns Snd',
-                0x0400 => 'BTV Digital',
-                0x0680 => 'VME VMPCM',
-                0x1000 => 'OLIGSM',
-                0x1001 => 'OLIADPCM',
-                0x1002 => 'OLICELP',
-                0x1003 => 'OLISBC',
-                0x1004 => 'OLIOPR',
-                0x1100 => 'LH Codec',
-                0x1400 => 'Norris',
-                0x1401 => 'ISIAudio',
-                0x1500 => 'Soundspace Music Compression',
-                0x2000 => 'DVM',
-                0x2001 => 'DTS Coherent Acoustics 2',
-                0x4143 => 'MPEG-4 (Divio)',
-            },
+            SeparateTable => 'RIFF AudioEncoding',
+            PrintConv => \%Image::ExifTool::RIFF::audioEncoding,
         },
         {
             Condition => '$self->{ASF_STREAM_TYPE} =~ /^(Video|JFIF|Degradable JPEG)$/',
@@ -623,7 +517,6 @@ sub GetGUID($)
 sub ProcessContentDescription($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 10;
@@ -634,7 +527,7 @@ sub ProcessContentDescription($$$)
         my $len = shift @len;
         next unless $len;
         return 0 if $pos + $len > $dirLen;
-        my $val = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$len),'II');
+        my $val = $exifTool->Decode(substr($$dataPt,$pos,$len),'UCS2','II');
         $exifTool->HandleTag($tagTablePtr, $tag, $val);
         $pos += $len;
     }
@@ -648,7 +541,6 @@ sub ProcessContentDescription($$$)
 sub ProcessContentBranding($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 40;
@@ -680,7 +572,7 @@ sub ReadASF($$$$$)
     my ($exifTool, $dataPt, $pos, $format, $size) = @_;
     my @vals;
     if ($format == 0) { # unicode string
-        $vals[0] = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$size),'II');
+        $vals[0] = $exifTool->Decode(substr($$dataPt,$pos,$size),'UCS2','II');
     } elsif ($format == 2) { # 4-byte boolean
         @vals = ReadValue($dataPt, $pos, 'int32u', undef, $size);
         foreach (@vals) {
@@ -705,7 +597,6 @@ sub ReadASF($$$$$)
 sub ProcessExtendedContentDescription($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 2;
@@ -718,7 +609,7 @@ sub ProcessExtendedContentDescription($$$)
         my $nameLen = unpack("x${pos}v", $$dataPt);
         $pos += 2;
         return 0 if $pos + $nameLen + 4 > $dirLen;
-        my $tag = Image::ExifTool::Unicode2Latin(substr($$dataPt,$pos,$nameLen),'v');
+        my $tag = Image::ExifTool::Decode(undef,substr($$dataPt,$pos,$nameLen),'UCS2','II','Latin');
         $tag =~ s/^WM\///; # remove leading "WM/"
         $pos += $nameLen;
         my ($dType, $dLen) = unpack("x${pos}v2", $$dataPt);
@@ -737,10 +628,9 @@ sub ProcessExtendedContentDescription($$$)
 # Process WM/Picture preview
 # Inputs: 0) ExifTool object reference, 1) dirInfo ref, 2) tag table reference
 # Returns: 1 on success
-sub ProcessPreview($$$)
+sub ProcessPicture($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart};
     my $dirLen = $$dirInfo{DirLen};
@@ -752,10 +642,10 @@ sub ProcessPreview($$$)
     my $n = $dirLen - 5 - $picLen;
     return 0 if $n & 0x01 or $n < 4;
     my $str = substr($$dataPt, $dirStart+5, $n);
-    if ($str =~ /^((?:..)*?)\0\0((?:..)*?)\0\0/) {
+    if ($str =~ /^((?:..)*?)\0\0((?:..)*?)\0\0/s) {
         my ($mime, $desc) = ($1, $2);
-        $exifTool->HandleTag($tagTablePtr, 1, $mime);
-        $exifTool->HandleTag($tagTablePtr, 2, $desc) if length $desc;
+        $exifTool->HandleTag($tagTablePtr, 1, $exifTool->Decode($mime,'UCS2','II'));
+        $exifTool->HandleTag($tagTablePtr, 2, $exifTool->Decode($desc,'UCS2','II')) if length $desc;
     }
     $exifTool->HandleTag($tagTablePtr, 3, substr($$dataPt, $dirStart+5+$n, $picLen));
     return 1;
@@ -768,7 +658,6 @@ sub ProcessPreview($$$)
 sub ProcessCodecList($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 20;
@@ -784,12 +673,12 @@ sub ProcessCodecList($$$)
         my $nameLen = Get16u($dataPt, $pos + 2) * 2;
         $pos += 4;
         return 0 if $pos + $nameLen + 2 > $dirLen;
-        my $name = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$nameLen),'II');
+        my $name = $exifTool->Decode(substr($$dataPt,$pos,$nameLen),'UCS2','II');
         $exifTool->HandleTag($tagTablePtr, "${type}Name", $name);
         my $descLen = Get16u($dataPt, $pos + $nameLen) * 2;
         $pos += $nameLen + 2;
         return 0 if $pos + $descLen + 2 > $dirLen;
-        my $desc = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$descLen),'II');
+        my $desc = $exifTool->Decode(substr($$dataPt,$pos,$descLen),'UCS2','II');
         $exifTool->HandleTag($tagTablePtr, "${type}Description", $desc);
         my $infoLen = Get16u($dataPt, $pos + $descLen);
         $pos += $descLen + 2 + $infoLen;
@@ -804,7 +693,6 @@ sub ProcessCodecList($$$)
 sub ProcessMetadata($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $verbose = $exifTool->Options('Verbose');
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 2;
@@ -817,7 +705,7 @@ sub ProcessMetadata($$$)
         my ($index, $stream, $nameLen, $dType, $dLen) = unpack("x${pos}v4V", $$dataPt);
         $pos += 12;
         return 0 if $pos + $nameLen + $dLen > $dirLen;
-        my $tag = Image::ExifTool::Unicode2Latin(substr($$dataPt,$pos,$nameLen),'v');
+        my $tag = Image::ExifTool::Decode(undef,substr($$dataPt,$pos,$nameLen),'UCS2','II','Latin');
         my $val = ReadASF($exifTool,$dataPt,$pos+$nameLen,$dType,$dLen);
         $exifTool->HandleTag($tagTablePtr, $tag, $val,
             DataPt => $dataPt,
@@ -840,13 +728,8 @@ sub ProcessASF($$;$)
     my $verbose = $exifTool->Options('Verbose');
     my $rtnVal = 0;
     my $pos = 0;
-    my ($buff, $err, @parentTable, @childEnd, %dumpParms);
+    my ($buff, $err, @parentTable, @childEnd);
 
-    if ($verbose > 2) {
-        $dumpParms{MaxLen} = 96 unless $verbose > 3;
-        $dumpParms{Prefix} = $$exifTool{INDENT};
-        $dumpParms{Out} = $exifTool->Options('TextOut');
-    }
     for (;;) {
         last unless $raf->Read($buff, 24) == 24;
         $pos += 24;
@@ -855,7 +738,7 @@ sub ProcessASF($$;$)
             # verify this is a valid ASF file
             last unless $tag eq '75B22630-668E-11CF-A6D9-00AA0062CE6C';
             my $fileType = $exifTool->{FILE_EXT};
-            $fileType = 'ASF' unless $fileType and $fileType =~ /^(ASF|WMV|WMA)$/;
+            $fileType = 'ASF' unless $fileType and $fileType =~ /^(ASF|WMV|WMA|DIVX)$/;
             $exifTool->SetFileType($fileType);
             SetByteOrder('II');
             $tagTablePtr = GetTagTable('Image::ExifTool::ASF::Main');
@@ -867,7 +750,18 @@ sub ProcessASF($$;$)
             last;
         }
         if ($size > 0x7fffffff) {
-            $err = 'Large ASF objects not supported';
+            if ($size > 0x7fffffff * 4294967296) {
+                $err = 'Invalid ASF object size';
+            } elsif ($exifTool->Options('LargeFileSupport')) {
+                if ($raf->Seek($size, 1)) {
+                    $exifTool->VPrint(0, "  Skipped large ASF object ($size bytes)\n");
+                    $pos += $size;
+                    next;
+                }
+                $err = 'Error seeking past large ASF object';
+            } else {
+                $err = 'Large ASF objects not supported (LargeFileSupport not set)';
+            }
             last;
         }
         # go back to parent tag table if done with previous children
@@ -875,7 +769,6 @@ sub ProcessASF($$;$)
             pop @childEnd;
             $tagTablePtr = pop @parentTable;
             $exifTool->{INDENT} = substr($exifTool->{INDENT},0,-2);
-            $dumpParms{Prefix} = $exifTool->{INDENT};
         }
         my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
         $verbose and $exifTool->VerboseInfo($tag, $tagInfo);
@@ -888,7 +781,7 @@ sub ProcessASF($$;$)
                         my $s = $$subdir{Size};
                         if ($verbose > 2) {
                             $raf->Read($buff, $s) == $s or $err = 'Truncated file', last;
-                            Image::ExifTool::HexDump(\$buff, undef, %dumpParms);
+                            $exifTool->VerboseDump(\$buff);
                         } elsif (not $raf->Seek($s, 1)) {
                             $err = 'Seek error';
                             last;
@@ -900,7 +793,6 @@ sub ProcessASF($$;$)
                         $pos += $$subdir{Size};
                         if ($verbose) {
                             $exifTool->{INDENT} .= '| ';
-                            $dumpParms{Prefix} = $exifTool->{INDENT};
                             $exifTool->VerboseDir($$tagInfo{Name});
                         }
                         next;
@@ -912,9 +804,7 @@ sub ProcessASF($$;$)
                         DirLen => $size,
                         DirName => $$tagInfo{Name},
                     );
-                    if ($verbose > 2) {
-                        Image::ExifTool::HexDump(\$buff, undef, %dumpParms);
-                    }
+                    $exifTool->VerboseDump(\$buff) if $verbose > 2;
                     unless ($exifTool->ProcessDirectory(\%subdirInfo, $subTable)) {
                         $exifTool->Warn("Error processing $$tagInfo{Name} directory");
                     }
@@ -928,7 +818,7 @@ sub ProcessASF($$;$)
         }
         if ($verbose > 2) {
             $raf->Read($buff, $size) == $size or $err = 'Truncated file', last;
-            Image::ExifTool::HexDump(\$buff, undef, %dumpParms);
+            $exifTool->VerboseDump(\$buff);
         } elsif (not $raf->Seek($size, 1)) { # skip the block
             $err = 'Seek error';
             last;
@@ -959,7 +849,7 @@ Windows Media Audio (WMA) and Windows Media Video (WMV) files.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
